use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/MooseX/AttributeHelpers.pm',
    'lib/MooseX/AttributeHelpers/Bool.pm',
    'lib/MooseX/AttributeHelpers/Collection/Array.pm',
    'lib/MooseX/AttributeHelpers/Collection/Bag.pm',
    'lib/MooseX/AttributeHelpers/Collection/Hash.pm',
    'lib/MooseX/AttributeHelpers/Collection/ImmutableHash.pm',
    'lib/MooseX/AttributeHelpers/Collection/List.pm',
    'lib/MooseX/AttributeHelpers/Counter.pm',
    'lib/MooseX/AttributeHelpers/Meta/Method/Curried.pm',
    'lib/MooseX/AttributeHelpers/Meta/Method/Provided.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/Array.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/Bag.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/Bool.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/Counter.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/Hash.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/ImmutableHash.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/List.pm',
    'lib/MooseX/AttributeHelpers/MethodProvider/String.pm',
    'lib/MooseX/AttributeHelpers/Number.pm',
    'lib/MooseX/AttributeHelpers/String.pm',
    'lib/MooseX/AttributeHelpers/Trait/Base.pm',
    'lib/MooseX/AttributeHelpers/Trait/Bool.pm',
    'lib/MooseX/AttributeHelpers/Trait/Collection.pm',
    'lib/MooseX/AttributeHelpers/Trait/Collection/Array.pm',
    'lib/MooseX/AttributeHelpers/Trait/Collection/Bag.pm',
    'lib/MooseX/AttributeHelpers/Trait/Collection/Hash.pm',
    'lib/MooseX/AttributeHelpers/Trait/Collection/ImmutableHash.pm',
    'lib/MooseX/AttributeHelpers/Trait/Collection/List.pm',
    'lib/MooseX/AttributeHelpers/Trait/Counter.pm',
    'lib/MooseX/AttributeHelpers/Trait/Number.pm',
    'lib/MooseX/AttributeHelpers/Trait/String.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/000_load.t',
    't/001_basic_counter.t',
    't/002_basic_array.t',
    't/003_basic_hash.t',
    't/004_basic_number.t',
    't/005_basic_list.t',
    't/006_basic_bag.t',
    't/007_basic_string.t',
    't/010_array_from_role.t',
    't/011_counter_with_defaults.t',
    't/012_basic_bool.t',
    't/020_remove_attribute.t',
    't/100_collection_with_roles.t',
    't/201_trait_counter.t',
    't/202_trait_array.t',
    't/203_trait_hash.t',
    't/204_trait_number.t',
    't/205_trait_list.t',
    't/206_trait_bag.t',
    't/207_trait_string.t',
    't/208_trait_bool.t',
    't/zzz-check-breaks.t',
    'xt/author/00-compile.t',
    'xt/author/kwalitee.t',
    'xt/author/mojibake.t',
    'xt/author/no-tabs.t',
    'xt/author/pod-coverage.t',
    'xt/author/pod-syntax.t',
    'xt/release/changes_has_content.t',
    'xt/release/distmeta.t',
    'xt/release/minimum-version.t',
    'xt/release/pod-no404s.t',
    'xt/release/portability.t'
);

notabs_ok($_) foreach @files;
done_testing;
